<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Confirmation</title>

</head>
<body>
    <h1>Login confirmation for {{web.host}}</h1>
    {% if denied %}
        <p>You denied access to log in into {{web.host}}</p>
    {% endif %}
    {% if not_accepted_scopes %}
        <p>{{web.host}} wants to access the information about:</p>
        <ul>
            {% for scope in not_accepted_scopes %}
                <li>{{ scope|title }}</li>
            {% endfor %}
        </ul>
    {% endif %}
    {% if accepted_scopes %}
        <p>List of previously accepted scopes:</p>
        <ul>
            {% for scope in accepted_scopes %}
                <li>{{ scope|title }}</li>
            {% endfor %}
        </ul>
    {% endif %}
    <form method="POST">
        {% csrf_token %}
        <button name="confirmation" value="true">Accept</button>
        <button name="confirmation" value="false">Deny</button>
    </form>
</body>
</html>
